package org.aohe.core.utils;

import cn.hutool.core.util.RuntimeUtil;
import cn.hutool.core.util.StrUtil;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class CommandUtils {

    public static void initSanedSocket(){
        String cmdResult = execForStrUtf8("systemctl status saned.socket");
        if (StrUtil.isNotEmpty(cmdResult)) {
            String status = CommandUtils.getSanedSocketStatus(cmdResult);
            log.info("saned.socket status：{}", status);
            if (StrUtil.isNotEmpty(status) && "dead".equals(status)) {

                execForStrUtf8("systemctl start saned.socket");
                status = CommandUtils.getSanedSocketStatus(execForStrUtf8("systemctl status saned.socket"));
                if("dead".equals(status)){
                    cmdResult = execForStrUtf8Sudo("systemctl start saned.socket");
                }
                log.info("start command {}", cmdResult);
                log.info("saned.socket successfully");
            }
        }
    }

    public static String execForStrUtf8(String command){
        return RuntimeUtil.execForStr(StandardCharsets.UTF_8,command);
    }

    public static String execForStrUtf8Sudo(String command){
        return RuntimeUtil.execForStr(StandardCharsets.UTF_8,"sudo " + command);
    }

    public static String getSanedSocketStatus(String commandResult) {
        String result = "";
        String[] resultArray = commandResult.split("\n");
        List<String> list = new ArrayList<>();
        for (String item : resultArray) {
            if (item.contains("Active:")) {
                String serialItem = item.replace("*", "").replace("\t", "").replace(" ", "");
                list.add(serialItem);
            }
        }
        if (!list.isEmpty()) {
            Pattern pattern = Pattern.compile("\\((.*?)\\)");
            Matcher matcher = pattern.matcher(list.getFirst());
            if (matcher.find()) {
                result = matcher.group(1);
            }
        }
        return result;
    }

    public static void killUsedPort(String port){
        List<LsofInfo> lsofInfos = findPort(port);
        for(LsofInfo lsofInfo : lsofInfos){
            log.info(" pro kill begin {}", lsofInfo);
            destroy(lsofInfo.getPid());
        }
    }

    public static List<LsofInfo> findPort(String port){
        String exec = StrUtil.format("lsof -i:{}",port);
        String s = execForStrUtf8Sudo(exec);
        log.info(s);
        return parseCommandOutput(s);
    }

    public static void destroy(String pid){
        String exec = StrUtil.format("kill -9 {}",pid);
        String s = execForStrUtf8Sudo(exec);
        log.info(s);
    }

    /**
     * lsof 格式化
     * @param output s
     * @return list
     */
    public static List<LsofInfo> parseCommandOutput(String output) {
        List<LsofInfo> processInfos = new ArrayList<>();
        String[] lines = output.split("\n");
        if(lines.length <= 1){
            return new ArrayList<>();
        }

        for (int i = 1 ; i < lines.length ; i ++) {
            //从第一行开始
            String[] parts = lines[i].trim().split("\\s+");
            if (parts.length < 6) {
                return new ArrayList<>();
            }
            LsofInfo processInfo = new LsofInfo();
            processInfo.setPid(parts[1]);
            processInfo.setType(parts[4]);

            processInfos.add(processInfo);
        }

        return processInfos;
    }

    @Data
    @ToString
    public static class LsofInfo {
        private String pid;
        private String type;
    }

}
